VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsLib_UndoRedo"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit
Private nUndo           As Long
Private nRedo           As Long
Private bEnabled        As Boolean
Private frm             As frmDoc
Private af              As AddFlow
Private bChanged        As Boolean
Private tempPath        As String

'Callback events
Public Event RefreshButtons()
Public Event CleanTagNode()
Public Event GetGraphConfig()
Public Event SaveGraphInfo()
Public Event ModeCommand(ByRef bModeCommand As Boolean)

Public Sub Init(oFrm As frmDoc)
Dim bModeCommand As Boolean
    Set frm = oFrm
    Set af = oFrm.AddFlow1
    RaiseEvent ModeCommand(bModeCommand)
    If bModeCommand Then Exit Sub
   
    CleanLastNameUndoFiles
    bEnabled = True 'After CleanLastNameUndoFiles
    nUndo = 0
    nRedo = 0
    CleanUndoFiles
    Save True
    RaiseEvent RefreshButtons
End Sub

Private Sub Class_Initialize()
    tempPath = SystemTempPath & TMP_SUBDIR
End Sub

''''''''''''''''''''''''''''''''''''''''''''''''''''''
'APIS to manage the change bit absolute and relative
''''''''''''''''''''''''''''''''''''''''''''''''''''''
Public Function GraphHasChanged() As Boolean
    GraphHasChanged = bChanged Or af.IsChanged
End Function
Public Sub StoreGraphChanges()
    If af.IsChanged Then
        bChanged = True
        af.SetChangedFlag False
        Save
    End If
End Sub
Public Sub ClearGraphChanges()
    bChanged = False
    af.SetChangedFlag False
End Sub
''''''''''''''''''''''''''''''''''''''''''''''''''''''
''''''''''''''''''''''''''''''''''''''''''''''''''''''

''''''''''''''''''''''''''''''''''''''''''''''''''''''
'APIS to manage the Undo Redo
''''''''''''''''''''''''''''''''''''''''''''''''''''''
Public Function CanUndo() As Boolean
    CanUndo = bEnabled And (nUndo > 1)
    If CanUndo Then
        CanUndo = Not af.readOnly
    End If
End Function

Public Function CanRedo() As Boolean
    CanRedo = bEnabled And (nRedo > 0)
    If CanRedo Then
        CanRedo = Not af.readOnly
    End If
End Function

Public Sub Undo()
    If Not CanUndo() Then Exit Sub
    nUndo = nUndo - 2
    nRedo = nRedo + 1
    Load
    nUndo = nUndo + 1
    RaiseEvent RefreshButtons
End Sub

Public Sub Redo()
    If Not CanRedo() Then Exit Sub
    nRedo = nRedo - 1
    Load
    nUndo = nUndo + 1
    RaiseEvent RefreshButtons
End Sub

Private Function UndoPath(file As String) As String
    If LenB(file) > 3 Then
        UndoPath = tempPath & Left$(file, 1) & Mid$(file, 3)
    Else
        bEnabled = False
    End If
End Function

Private Sub Load()
Dim fileName As String
    
    With frm
        fileName = UndoPath(FileSinExtension(.sFullDocname)) & "." & Format$(nUndo, "000000") & "." & FILE_UNDO_EXT
    
        ' Erase the present addflow
        af.Nodes.Clear ' All including the "Logical = False"
        #If DEBUG_UNDO Then
            af.LogicalOnly = True
            Debug.Assert af.Nodes.count = 0
            af.LogicalOnly = False
        #End If
        
        On Error GoTo ControlError   ' Activa la rutina de control de errores.
        af.LoadFile fileName
        On Error GoTo 0   ' Desactiva la deteccin de errores.
        #If DEBUG_ExtendLogOut Then
            Logout " clsUndoRedo.Load: " & fileName
        #End If
        
        ' Lee la informacion del grafo
        RaiseEvent GetGraphConfig
        If nUndo = 0 Then
            ClearGraphChanges
        Else
            af.SetChangedFlag True
        End If
    End With
    
Exit Sub   ' Salir para evitar el controlador.
Dim iReintentos As Long
ControlError:   ' Rutina de control de errores.
    Logout " clsUndoRedo.Load: af.LoadFile " & fileName & ", Error #" & Str$(Err.Number) & ": " & Err.Description
    Select Case Err.Number   ' Evala el nmero de error.
        Case &HF108&, 438         '&HF108&: No es un fichero FSM; 438: El objeto no acepta esta propiedad
            iReintentos = iReintentos + 1
            If iReintentos < 50 Then
                Logout "  reintento" & Str$(iReintentos)
                DoEvents
                Resume ' Continuar ejecucin en la lnea que caus el error.
            End If
    End Select
    bEnabled = False
    Exit Sub
End Sub


Public Sub Save(Optional forceSave As Boolean = False)
Dim fileName        As String
Dim sPath           As String
Dim bModeCommand    As Boolean
    
    RaiseEvent ModeCommand(bModeCommand)
    If bModeCommand Then Exit Sub
    With frm
        If Not forceSave And (af.readOnly Or Not bEnabled) Then Exit Sub

        fileName = UndoPath(FileSinExtension(.sFullDocname)) & "." & Format$(nUndo, "000000") & "." & FILE_UNDO_EXT
        nUndo = nUndo + 1

        'Store the diagram info into a special node
        RaiseEvent SaveGraphInfo
        
        On Error GoTo ControlError   ' Activa la rutina de control de errores.
        'If the name is used, first kill it
        If LenB(Dir(fileName, vbNormal + vbReadOnly + vbHidden + vbSystem)) Then
            SetAttr fileName, vbNormal
            Kill fileName
        End If
        
        sPath = RutaSinFile(fileName)
        If Not FileExist(sPath, vbDirectory) Then
            CreateNewDirectory sPath
        End If

        af.SaveFile fileName
        On Error GoTo 0   ' Desactiva la deteccin de errores.
        #If DEBUG_ExtendLogOut Then
            Logout " clsUndoRedo.Save: " & fileName
        #End If
        
        'Elimina el nodo oculto empleado para salvar global info
        RaiseEvent CleanTagNode
        If (nUndo = 2) Or (nRedo > 0) Then
            nRedo = 0
            RaiseEvent RefreshButtons
        End If
    End With
Exit Sub   ' Salir para evitar el controlador.
Dim iReintentos As Long
ControlError:   ' Rutina de control de errores.
    Select Case Err.Number   ' Evala el nmero de error.
        Case &H80010108     'Error de automatizacin
            iReintentos = iReintentos + 1
            If iReintentos < 10 Then
                Logout " clsUndoRedo.Save: af.SaveFile " & fileName & " reintento" & Str$(iReintentos)
                Resume
            Else
                Logout " clsUndoRedo.Save, Error #0x80010108 : " & Err.Description
            End If
        Case Else
            Logout " clsUndoRedo.Save, Error #" & Str$(Err.Number) & ": " & Err.Description
    End Select
    bEnabled = False
    Resume Next  ' Continuar ejecucin en la lnea siguiente a la que caus el error.
End Sub

Public Sub CleanUndoFiles()
Dim bModeCommand As Boolean
    RaiseEvent ModeCommand(bModeCommand)
    If bModeCommand Then Exit Sub
    CleanFiles UndoPath(frm.sFullDocname)
End Sub


Private Sub CleanLastNameUndoFiles()
Dim bModeCommand As Boolean
    RaiseEvent ModeCommand(bModeCommand)
    If bModeCommand Then Exit Sub
    CleanFiles UndoPath(frm.sLastFullDocname)
    frm.sLastFullDocname = vbNullString
End Sub

Private Sub CleanFiles(fileWithPath As String)
Dim file As String
Dim ruta As String
    If LenB(fileWithPath) Then
        ruta = RutaSinFile(fileWithPath)
        If Right$(ruta, 1) <> "\" Then ruta = ruta & "\"
        On Error GoTo ControlError   ' Activa la rutina de control de errores.
        file = Dir$(FileSinExtension(fileWithPath) & ".??????." & FILE_UNDO_EXT, vbNormal + vbReadOnly + vbHidden + vbSystem)
        Do While LenB(file)
            file = ruta & file
            'Debug.Print "Erasing: " & file
            SetAttr file, vbNormal
            Kill file
            file = Dir$()
        Loop
        On Error GoTo 0   ' Desactiva la deteccin de errores.
    End If
Exit Sub   ' Salir para evitar el controlador.
ControlError:   ' Rutina de control de errores.
    Select Case Err.Number   ' Evala el nmero de error.
        Case 53
        Case Else
            Logout " clsUndoRedo.CleanFiles, Error #" & Str$(Err.Number) & ": " & Err.Description
    End Select
    bEnabled = False
    Resume Next  ' Continuar ejecucin en la lnea siguiente a la que caus el error.
End Sub
